 /***********************************************************************
 *                                                                      *
 * main.c                                                               *
 * Sound & Play command test by TK      		                       	*
 * History : 2010/04/15 	start                                   *
 ***********************************************************************/
#include <string.h>
#include "osc.h"
#include "t16.h"
#include "lcd.h"
#include "port.h"
#include "init.h"
#include "lcdManager.h"

/*proto type*/
void t16ch0Int(void)__attribute__ ((interrupt_handler));
void t16ch1Int(void)__attribute__ ((interrupt_handler));
void t16ch2Int(void)__attribute__ ((interrupt_handler)); //**TK**
void p0xInt(void)__attribute__ ((interrupt_handler));
void timer1init(int); //**TK**
void display2lcd(void)__attribute__ ((interrupt_handler)); //**TK**
void Wait(void);
void hardware_init(void);
int main(void);
int random(void); //**TK**
void setNum(unsigned int,unsigned int); //**TK**

void setSound(int,int); //**TK**
void initPlay(int); //**TK**
int play(int,unsigned char *);//**TK**
void playWait(int); //**TK**
void playOnOff(int);


/* Global variable define */
 volatile unsigned char t16ch0Flg;
 volatile unsigned char t16ch2Flg; //**TK**
 volatile unsigned char sw2Flg;
 volatile unsigned char sw3Flg;
 volatile unsigned int random_seed;			// **TK** valuable for semi-random function

 /* work for non-grayscale text screen */
 volatile unsigned char buf[12*4]; 			// **TK** virtual screen buffer
 volatile unsigned char line_buf[13];		// **TK** line buffer to call displayStr()

 /* work for grayscale screen */
 volatile unsigned char temp_screen[80*5*3]; /* 3 planes for 4 gray scales */
 volatile int lcd_counter ;


/* valuables for play function */
 volatile unsigned int tone_define[35];  //**TK**
 volatile unsigned int length_define[7];  //**TK**

 /* valuables for BGM function */
volatile unsigned int BGM_buffer_freq[128];
volatile unsigned char BGM_buffer_length[128];
volatile unsigned char BGM_buffer_volume[128];
volatile unsigned int FGM_buffer_freq[16];
volatile unsigned char FGM_buffer_length[16];
volatile unsigned char FGM_buffer_volume[16];

volatile int BGM_FGM_tempo;
volatile int FGM_going;

volatile int BGM_play_index;
volatile int BGM_play_freq;
volatile int BGM_play_volume;
volatile int BGM_play_count;

volatile int FGM_play_index;
volatile int FGM_play_freq;
volatile int FGM_play_volume;
volatile int FGM_play_count;


/************************************************************************
 *   main function.
 *   param    :     void
 *   return   :     none
 *   history  :     2009/01/14	start
 ***********************************************************************/
int main(void) {

	int i,ret;

	controlPsc(PSC_PRUN_RUN);        			//prescaler run
	setClockGear(CLG_CCLKGR_1);      			//set clock gear *defalt=0x0

	//IOSC->OSC3
	SET_OSC3_WAIT_CYCLE(OSC3_WAIT_CYCLE_256);
	changeSysClkIoscTo3();
	hardware_init();

	//clear LCD
	clearAllLcdLine();
	Wait();

	// init play function

	initPlay(200);

	while (1){

		//Init display
		clearAllLcdLine();
		displayStr(0,"BGM test");
		displayStr(1,"          ");
		displayStr(2,"by T.K");
		displayStr(3,"**push sw2**");
		reverseStr(0);

		//SW2 wait
		sw2Flg = 0;
		while(sw2Flg == 0){}; sw2Flg = 0;
		//Sound testing
		clearAllLcdLine();
		displayStr(0,"BGM sample");
		displayStr(3,"============");
		/* BGM data setting */
		ret = play(0,"V2O4L3CR2O3GR2ERRARB2BbAARL3GO4EGL2AL3FL2GL3ERCRDL2O3B");
		sw3Flg = 0;
		while (1){
			for (i=11;i>=0;i--) {
				if (sw3Flg ==1){
					/* FGM inserting */
					displayStr(0,"FGM inserted");
					ret=play(1,"V2O4L6GL5BL6G");
					displayChar(2,i+1,32);
					displayChar(1,i+1,91+32);
					Wait();Wait();Wait();
					displayChar(1,i+1,32);
					sw3Flg = 0;
					displayStr(0,"BGM returned");
				}
				displayChar(2,i,91+32+(i % 2));
				displayChar(2,i+1,32);
				Wait();Wait();
			}
			displayChar(2,0,0);
		}
	}
	return 1;
}



 /************************************************************************
 *   t16ch1Int function.
 * 	 t16Ch1 interrupt routine.
 *   param    :     void
 *   return   :     none
 *   history  :     2009/01/14   start
 ***********************************************************************/
void t16ch1Int(void) {

	resetT16Ch1IF();
	P2_OUT ^= 0x80;			//LED ON/OFF

}

/************************************************************************
 *   t16ch0Int function.
 * 	 t16Ch0 interrupt routine.
 *   param    :     void
 *   return   :     none
 *   history  :     2009/01/14   start
 ***********************************************************************/
void t16ch0Int(void) {
	resetT16Ch0IF();
	t16ch0Flg = 1;
 }

/************************************************************************
 *   t16ch2Int function.
 * 	 t16Ch2 interrupt routine. Plays BGM and FGM.
 *   param    :     void
 *   return   :     none
 *   history  :     2010/04/16   start by **TK**
 ***********************************************************************/
void t16ch2Int(void) {

	resetT16Ch2IF();

	// If FGM playing
	if (FGM_going == 1){
		if (FGM_play_count == 0){
			// check if end of FGM
			if (FGM_buffer_freq[FGM_play_index] == 0xffff){
				setSound(0,1);
				FGM_going = 0;
			}
			// Play next FGM data
			FGM_play_count = FGM_buffer_length[FGM_play_index];
			FGM_play_freq = FGM_buffer_freq[FGM_play_index];
			FGM_play_volume = FGM_buffer_volume[FGM_play_volume];
			setSound(FGM_play_freq,FGM_play_volume);
			FGM_play_index ++;
		}else{
			// Playing current FGM data
			FGM_play_count --;
		}
	}
	// BGM playing
	if (BGM_play_count == 0){
		if (BGM_buffer_freq[BGM_play_index] == 0xffff){
			setSound(0,1);
			BGM_play_count = 8;
			BGM_play_index = 0;
		}
		// Play next BGM data
		BGM_play_count = BGM_buffer_length[BGM_play_index];
		BGM_play_freq = BGM_buffer_freq[BGM_play_index];
		BGM_play_volume = BGM_buffer_volume[BGM_play_volume];
		// If FGM is on going, hold the BGM sound.
		if (FGM_going == 0) {
			setSound(BGM_play_freq,BGM_play_volume);
		}
		BGM_play_index ++;
	}else{
		// Playing current BGM data
		BGM_play_count --;
	}


 }

 /************************************************************************
 *	 p0xInt function.
 *   P0X interrupt routine.
 *   param    :     void
 *   return   :     int
 *   history  :     2009/01/14	start
 ***********************************************************************/
 void p0xInt(void) {
	/* interrupt flag*/
	char intFlag = 0;
	//Check Interrupt status
	if(isP06Int() == P_INT_OCCURED) {
		//P06 intterupt occured
		intFlag |= P_BIT_X6;
		sw2Flg = 1;
	}
	if(isP07Int() == P_INT_OCCURED) {
		//P07 intterupt occured
		intFlag |= P_BIT_X7;
		sw3Flg = 1;
	}
	//Interrupt Flag clear
	resetP0IF((unsigned char) intFlag);
 }

 /************************************************************************
  *   Transfer virtual screen data to LCD display at LCD interrupt
  *   param    :     void
  *   return   :     none
  *   history  :     2010/03/17	start
  ***********************************************************************/
 void display2lcd(void){		//**TK**
	 switch (lcd_counter) {
		case 0:
			 displayTempScreen((char *)temp_screen + 400 *2 );			//**TK**
			 break;
		case 1:
			 displayTempScreen((char *)temp_screen + 400);			//**TK**
			 break;
		case 2:
			 displayTempScreen((char *)temp_screen);			//**TK**
			 break;
	 }

	 lcd_counter ++;
	 if (lcd_counter > 2) {
		 lcd_counter = 0;
	 }
 	resetLcdIF();				//**TK**
 }

 /************************************************************************
  *   Generating semi-random number
  *   param    :     void
  *   return   :     0-255 random number
  *   history  :     2010/03/17	start
  ***********************************************************************/
 int random(void){								//**TK**
 	random_seed = (random_seed * 113 + 7);		//**TK**
 	return (random_seed % 0x100);				//**TK**
 }

 /************************************************************************
   *   Display
   *   param    :     num : unsigned int value to be displayed
   *   				  loc : column location in line_buffer
   *   return   :     none
   *   history  :     2010/03/17	start
   ***********************************************************************/
  void setNum(unsigned int loc,unsigned int num){	//**TK**
	unsigned int i,cmp;							//**TK**
  	i = 0 ;										//**TK**
  	cmp = 10000;
  	for (i = 0; i < 5; i++) {
  		line_buf[loc+i] = num / cmp + 0x30;
  		num = num % cmp;
  		cmp /= 10;
  	}
  	line_buf[loc+i] ='\0';
  }

 /************************************************************************
 *   Wait function.
 *   wait.
 *   param    :     void
 *   return   :     none
 *   history  :     2009/01/14	start
 ***********************************************************************/
void Wait(void) {

	t16ch0Flg = 0;
	//start T16CH0 timer
	startT16Ch0();

	//T16CH0 timer interrupt enable
	startT16Ch0Int();

	while(t16ch0Flg != 1){
	    asm("halt");
		asm("nop");
    }
}
/************************************************************************
 *   hardware init function.
 *   software wait.
 *   param    :     void
 *   return   :     none
 *   history  :     2009/01/14	start
 ***********************************************************************/
void hardware_init(void) {

	InitParamT16 initParam;

	/*---------------------------------------------*/
	//P06,P07 = INPUT
	setPortInput(0xC0, 0x00, 0x00, 0x00);
	//P27 = OUTPUT
	setPortOutput(0x00, 0x00, 0x80, 0x00);
	contorlP0IputOn(0xC0);
	initP0Int(0x03);
 	setP0IntEdgeDown(0xC0);
	setP0Chat4to7(P0_CAHT_TIME_PCLK_8192);
	startP0Int(0xC0);

	/*---------------------------------------------*/
	// T16E setting for sound function

	// TOUT4 output enable
	P0_PMUX_P04MUX 	= 1;			// TOUT4 select
	P0_IO_P0IO 		|= 0x10;		// P4 output enable
	P0_IEN_P0IEN 	&= 0xEF;		// P4 input disable

	// T16E ch1 setting
	T16E_CLK1_T16EDF = 0x07;		// CLK = PCLK/256 = about 31.250KHz
	T16E_CTL1_INITOL = 0;			// Initial level = LOW
	T16E_CTL1_SELFM = 1;			// fine mode select
	T16E_CTL1_CBUFEN = 1;			// Comparison enable
	T16E_CTL1_INVOUT = 1;			// No invert output
	T16E_CTL1_CLKSEL = 0;			// internal clock select
	T16E_CTL1_OUTEN = 1;			// Clock output enabled
	T16E_CTL1_T16ERST = 1;			// counter reset
	T16E_CTL1_T16ERUN = 0;			// Timer stop

	T16E_INT1_CBIE = 0;				// No interrupt for counter B
	T16E_INT1_CAIE = 0;				// No interrupt for counter A

	/*---------------------------------------------*/
	// T16 CH2 setting for play function
    initParam.clkMode = T16_CLK_MODE_IN;		//internal
	initParam.cntMode = T16_CNT_MODE_REPEAT;	//oneshot
	initParam.cntClock = T16_IN_CLOCK_1024;     //1/8192
	initParam.exActiveLv = T16_EX_ACTIVE_LV_UP;	//Polar 1:Rise
	initParam.count = 0x0080;	                //**TK**
    initT16Ch2(&initParam);
	initT16Ch2Int(0x03);
	stopT16Ch2();
	stopT16Ch2Int();

	/*---------------------------------------------*/
	//T16 CH0 setting(wait 0.25s setting)
    initParam.clkMode = T16_CLK_MODE_IN;		//internal
	initParam.cntMode = T16_CNT_MODE_ONESHOT;	//oneshot
	initParam.cntClock = T16_IN_CLOCK_8192;     //1/8192
	initParam.exActiveLv = T16_EX_ACTIVE_LV_UP;	//Polar 1:Rise
	initParam.count = 0x0080;	                //**TK**

    initT16Ch0(&initParam);
	initT16Ch0Int(0x03);

	/*---------------------------------------------*/
	//T16 CH1 setting(moved by TK)
	timer1init(3);

	/*---------------------------------------------*/
	// initializes LCD
	initLcdPower(LCD_PWR_BOOSTER_OFF, LCD_HEAVY_PROTECT_OFF);
	startLcdClk(LCD_CLK_SELECT_HSCLK, LCD_CLK_DIV_32);
	initLcd(LCD_DUTY_32, LCD_CONTRAST_LV_10);
	reverseLcdCom();
	reverseLcdSeg();
	initLcdInt(0x03);		//**TK**
	lcdOn();
}

void timer1init(int timer_value) {

	InitParamT16 initParam;

	stopT16Ch1();
	stopT16Ch1Int();
    initParam.clkMode = T16_CLK_MODE_IN;		//internal
	initParam.cntMode = T16_CNT_MODE_REPEAT;	//repeat
	initParam.cntClock = T16_IN_CLOCK_8192;     //1/8192
	initParam.exActiveLv = T16_EX_ACTIVE_LV_UP;	//Polar 1:Rise
	//initParam.count = 0x01e7 ;	                //reload data 487
	initParam.count = 0x01e7 /3 * timer_value ;	                //**TK**
    initT16Ch1(&initParam);
	initT16Ch1Int(0x03);
	startT16Ch1();
	startT16Ch1Int();
}

/*************************************************************
 * setSound function.
 * This function make sound
 *
 * param: frequency : frequency of sound in HZ
 *        volume : 1 = low / 2 = high
 * return: none
 * history: 2010/04/15	start
 ************************************************************/

void setSound(int frequency,int volume) {
	int work;
	if ((frequency>0)&&(frequency<=8000)&&(volume >= 1)&&(volume <= 2)){
		work = 31250/frequency;

		T16E_CB1_T16ECB = work;
		if (volume == 1) {
			T16E_CA1_T16ECA = 1;
		}else{
			T16E_CA1_T16ECA = work/2;
		}
		T16E_CTL1_T16ERST = 1;			// counter reset
		T16E_CTL1_T16ERUN = 1;			// Timer start
	}
	if (frequency == 0){
		T16E_CTL1_T16ERST = 1;			// counter reset
		T16E_CTL1_T16ERUN = 0;			// Timer stop
	}
}
/*************************************************************
 * initPlay function.
 * This function initializes play functions
 *
 * param: tempo : tempo in BPM
 * return: none
 * history: 2010/04/15	start
 ************************************************************/
void initPlay(int tempo) {

	int i;

	// tone - frequency definition
	tone_define[14]=262;
	tone_define[15]=294;
	tone_define[16]=330;
	tone_define[17]=349;
	tone_define[18]=392;
	tone_define[19]=440;
	tone_define[20]=494;

	for (i=0; i<=6 ; i++) {
		tone_define[i] = tone_define[14+i]/4;
		tone_define[7+i] = tone_define[14+i]/2;
		tone_define[21+i] = tone_define[14+i]*2;
		tone_define[28+i] = tone_define[14+i]*4;
	}

	// length for each onpu (basing 1/32 = 1)

	length_define[0] = 64;
	length_define[1] = 32;
	length_define[2] = 16;
	length_define[3] = 8;
	length_define[4] = 4;
	length_define[5] = 2;
	length_define[6] = 1;

	// tempo setting

	BGM_FGM_tempo = tempo;
	FGM_going = 0;

}

/*************************************************************
 * Play function.
 * This function analize Pseude-MML(Music Macro Language) and stores
 *       data for Timer2 interrupt handler
 * param:  int BGM_FGM : 0 = FGM, 1= BGM
 *         unsinged char*  str     MML String(0-255 charactors).
 * return: result (0=GOOD, 1=Error)
 * history: 2010/04/15	start
 ************************************************************/

int play(int BGM_FGM,unsigned char *str) {
	int  i; 		// loop counter
	int  store_index = 0; // index for BGM_bufer
	int huten;
	int play_leng_temp;
	int play_freq = 0;
	int play_tone = 0;
	int	play_oct = 3;
	int	play_vol = 2;
	int play_leng = 4;

	playOnOff(0);

	// Select Index for FGM/BGM

	if (BGM_FGM == 0) {
		BGM_play_index = 0;
		BGM_play_count = 0;
	}else{
		FGM_play_index = 0;
		FGM_play_count = 0;
		FGM_going = 1;
	}

	for(i = 0; i < 256; i++) {

		// handling end of MML data

		if ((*(str + i) == '\0') || ((store_index >=127)&&(BGM_FGM == 0)) || ((store_index >=15)&&(BGM_FGM == 1))) {
			if (BGM_FGM == 0) {
				BGM_buffer_freq[store_index] = 0xffff;
				BGM_buffer_length[store_index] = 0;
				BGM_buffer_volume[store_index] = 0;
			}else{
				FGM_buffer_freq[store_index] = 0xffff;
				FGM_buffer_length[store_index] = 0;
				FGM_buffer_volume[store_index] = 0;
			}
			break;	// end of String.
		}

		huten = 0;
		play_leng_temp = play_leng;

		// analizing the string

		switch (*(str+i)) {

			// tone code processing

			case 'C' :
			case 'D' :
			case 'E' :
			case 'F' :
			case 'G' :
			case 'A' :
			case 'B' :
				    play_tone = ((int)*(str+i)) - 0x41;
				    if (play_tone <= 1) {
				    	play_tone += 5;			// for AB
				    }else{
				    	play_tone -= 2;			// for CDEFG
				    }
				    play_freq = tone_define[(play_oct -1) *7 + play_tone];

				    // length option handling

					switch (*(str+i+1)) {
						case  '0':
						case  '1':
						case  '2':
						case  '3':
						case  '4':
						case  '5':
						case  '6':
							play_leng_temp = ((int)*(str+i+1)) - 0x30;
							i++;

							// added to handle huten after length option

							if (*(str+i+1) =='.'){
								huten = 1;
								i++;
							}
							break;

						// HUTEN handling

						case  '.':
							huten = 1;
							i++;
							break;

						// #,b handling

						case  '#':
						case  'b':
							if (*(str+i+1) == '#') {
								play_freq = (play_freq + tone_define[(play_oct -1) *7 + play_tone + 1])/2;
							}else{
								play_freq = (play_freq + tone_define[(play_oct -1) *7 + play_tone - 1])/2;
							}
							i++;

							// length option after #/b handling

							switch (*(str+i+1)) {
								case  '0':
								case  '1':
								case  '2':
								case  '3':
								case  '4':
								case  '5':
								case  '6':
									i++;
									play_leng_temp = ((int)*(str+i+1)) - 0x30;

									// added to handle huten after #,b + length option

									if (*(str+i+1) == '.'){
										huten = 1;
										i++;
									}
									break;
								case  '.':
									i++;
									huten = 1;
									break;
							}
					}

					// storing analyzed note data to BGM or FGM buffer

					if (BGM_FGM ==0){
						BGM_buffer_freq[store_index] = play_freq;
						BGM_buffer_length[store_index] = length_define[play_leng_temp] + huten * length_define[play_leng]/2;
						BGM_buffer_volume[store_index] = play_vol;
					}else{
						FGM_buffer_freq[store_index] = play_freq;
						FGM_buffer_length[store_index] = length_define[play_leng_temp] + huten * length_define[play_leng]/2;
						FGM_buffer_volume[store_index] = play_vol;
					}
					store_index++;
					huten = 0;
					break;

			// rest handling

			case 'R' :
					switch (*(str+i+1)) {
						case  '0':
						case  '1':
						case  '2':
						case  '3':
						case  '4':
						case  '5':
						case  '6':
							play_leng_temp = ((int)*(str+i+1)) - 0x30;
							i++;

							// added to handle huten after length option.

							if (*(str+i+1) =='.'){
								huten = 1;
								i++;
							}
							break;
						case  '.':
							i++;
							huten = 1;
							break;
					}

					// storing rest data into BGM or FGM bugger

					if (BGM_FGM ==0){
						BGM_buffer_freq[store_index] = play_freq;
						BGM_buffer_length[store_index] = length_define[play_leng_temp] + huten * length_define[play_leng]/2;
						BGM_buffer_volume[store_index] = play_vol;
					}else{
						FGM_buffer_freq[store_index] = play_freq;
						FGM_buffer_length[store_index] = length_define[play_leng_temp] + huten * length_define[play_leng]/2;
						FGM_buffer_volume[store_index] = play_vol;
					}
					store_index++;
					break;

			// Octave change handling

			case 'O' :
					switch (*(str+i+1)) {
						case  '1':
						case  '2':
						case  '3':
						case  '4':
						case  '5':
							play_oct = ((int)*(str+i+1)) - 0x30;
							i++;
							break;
					}
					break;

			// volume control handling

			case 'V' :
					switch (*(str+i+1)) {
						case  '1':
						case  '2':
							play_vol = ((int)*(str+i+1)) - 0x30;
							i++;
							break;
					}
					break;

			// length setting handling

			case 'L' :
					switch (*(str+i+1)) {
						case  '0':
						case  '1':
						case  '2':
						case  '3':
						case  '4':
						case  '5':
							play_leng = ((int)*(str+i+1)) - 0x30;
							i++;
							break;
					}
					break;
		}
	}
	playOnOff(1);
	return 1;
}

/************************************************************************
*   Play ON/OFF function.
*   wait.
*   param    :     1 = ON, 0 = OFF
*   return   :     none
*   history  :     2010/04/16	start
***********************************************************************/
void playOnOff(int ON_OFF) {

	InitParamT16 initParam;

	if (ON_OFF ==1){
		//set T16CH2 timer
	    initParam.clkMode = T16_CLK_MODE_IN;		//internal
		initParam.cntMode = T16_CNT_MODE_REPEAT;	//oneshot
		initParam.cntClock = T16_IN_CLOCK_1024;     //1/8192
		initParam.exActiveLv = T16_EX_ACTIVE_LV_UP;	//Polar 1:Rise
		initParam.count = 3750 / BGM_FGM_tempo * 8;	      // value for 1/64 length
	    initT16Ch2(&initParam);
		initT16Ch2Int(0x03);

		// enable interval timer for play BGM/FGM

		startT16Ch2();
		startT16Ch2Int();

	}else{

		T16E_CTL1_T16ERST = 1;			// counter reset
		T16E_CTL1_T16ERUN = 0;			// sound source stop

		// disable interval timer for play BGM/FGM

		stopT16Ch2();
		stopT16Ch2Int();

	}
}
